#include "cameraReader.h"

#include "global.h"

CameraReader::CameraReader()
{
	strcpy_s(wspace," \t\n");
	filmback[0] = filmback[1] = 0.0f;
}

CameraReader::~CameraReader()
{
	while(!cameras.empty())
	{
		delete (CamEntry*) cameras.back();
		cameras.pop_back();
	}
}

void CameraReader::parse( const char* filename )
{
	FILE* in = fopen( filename, "r" );
	if( in == NULL )
	{
		printf("Could not open file %s\n", filename );
		return;
	}

	// parse the file
	const int BUFSIZE = 256;
	char buffer[BUFSIZE];
	char *token;
	parseError = false;
	lineNumber = 0;

	while ( !feof(in) && !parseError ) 
	{
		if( !fgets( buffer, BUFSIZE, in ) )
			continue;
		lineNumber++;
		token = strtok( buffer, wspace);

		if( token == NULL || token[0] == '#' )
			continue;
		else if( strcmp( token, "filmback" ) == 0 )
		{
			token = strtok( NULL, wspace );
			filmback[0] = (float)atof( token );
			token = strtok( NULL, wspace );
			filmback[1] = (float)atof( token );
		}
		else if( strcmp( token, "frame" ) == 0 )
		{
			token = strtok( NULL, wspace );	// read frame number; not necessary
			cameras.push_back(new CamEntry);
		}
		else if( strcmp( token, "transform" ) == 0 )
		{
			for(int i=0;i<4;i++)
			{
				if( !fgets( buffer, BUFSIZE, in ) )
				{
					parseError = true;
					continue;
				}
				CamEntry *lastEntry = cameras.back();
				for(int j=0;j<4;j++)
				{
					token = strtok( j == 0 ? buffer : NULL, wspace );
					lastEntry->m[i*4+j] = (float)atof(token)*(i == 3 ? MASSIVE_PRIME_SCALE_FACTOR*WORLD_SCALE_FACTOR : 1);
				}
			}
		}
		else if( strcmp( token, "fov" ) == 0 )
		{
			CamEntry *lastEntry = cameras.back();
			token = strtok( NULL, wspace );
			lastEntry->fov = (float)atof(token);
		}
		else
		{

		}
	}

	fclose(in);

}
